from mpimip import Communicator, initialization

# Create a communicator object
communicator = Communicator()


# Here we'll create a function that will generate a dummy file to do the demonstration.
# We'll also use the initialization decorator because we want it only to happen in the root process.
@initialization
def create_example_file(filename):
    with open(filename, "w") as f:
        for i in range(100):
            f.write("%i\n" % i)
    return True


# Create test file
test_filename = "test.txt"
create_example_file(test_filename)


@initialization
def read_file_contents(filename):
    with open(filename, "r") as f:
        return f.read()


# Read the file just in the root process:
data = read_file_contents(test_filename)

if data is not None:
    # Check what do the different processes have
    print("I'm %s and I have something" % (str(communicator.rank)))
else:
    # Check what do the different processes have
    print("I'm %s and I don't have anything" % (str(communicator.rank)))

# Now we can distribute the information using the communicator method bcast

data = communicator.bcast(data)


if data is not None:
    # Check what do the different processes have
    print("After the broadcast I'm %s and I have something" % (str(communicator.rank)))
else:
    # Check what do the different processes have
    print("After the broadcast I'm %s and I don't have anything" % (str(communicator.rank)))


"""
Running with 8 tasks the output will look like:

--------------------------------------------------------------------------
I'm 2 and I don't have anything
I'm 1 and I don't have anything
I'm 3 and I don't have anything
I'm 4 and I don't have anything
I'm 6 and I don't have anything
I'm 7 and I don't have anything
I'm 5 and I don't have anything
I'm 0 and I have something
After the broadcast I'm 4 and I have something
After the broadcast I'm 0 and I have something
After the broadcast I'm 1 and I have something
After the broadcast I'm 5 and I have something
After the broadcast I'm 3 and I have something
After the broadcast I'm 7 and I have something
After the broadcast I'm 2 and I have something
After the broadcast I'm 6 and I have something
--------------------------------------------------------------------------
"""
